/******************************************************************************
*                                                                             *
*                            Module specification                             *
*                                                                             *
*                                Copyright by                                 *
*                                                                             *
*                              Azoteq (Pty) Ltd                               *
*                          Republic of South Africa                           *
*                                                                             *
*                           Tel: +27(0)21 863 0033                            *
*                          E-mail: info@azoteq.com                            *
*                                                                             *
*******************************************************************************
Name             :  lower level PIC18F4550.c
Description      :  PIC specific functions for IQS316 I2C Firmware library
*******************************************************************************/

#include "includes.h"

//*****************************************************************************
//
//! Initialise
//!
//! Initializes the PIC18F4550
//!
//! \param None
//!
//! \return None
//
//*****************************************************************************
void
init(void)
{
    LATB = 0x00;                //PORTB all outputs - low
    TRISB = 0x00;               //IQS316 MCLR on B2 - hold in reset

    TRISD = 0x00;               //configure PORTD for output
    LATD = 0x0F;                //LEDS off

    LATC = 0x00;                //start by setting debug io
    TRISC = 0xF8;               //RC0-RC2 used for debug io's

    ADCON1 = 0x0F;              //PORTA all digital operation

    LATA = 0x00;
    TRISA = 0x00;               //All outputs

    INTCON2bits.RBPU = 0;       //For PIC18LF4550-Eval-Rev4A hardware with 74HC573 latch
    TRISCbits.TRISC0 = 0;
    LATCbits.LATC0  = 1;        //un-latch 74HC573 to make OUTD follows LATD
    TRISCbits.TRISC1 = 0;       //enable the latch
    LATCbits.LATC1 = 0;

    Comms_init();
    //
    // Add delay here to allow IQS316 to startup
    // According to datasheet first comms is available roughly 16ms
    // after MCLR is released
    //
    DelayMS(15);

    // Place other functions responsible for hardware initialization here.

    IQS316_Settings();
}

//*****************************************************************************
//
//! I2C Initialise
//!
//! Initializes the I2C module on the PIC18F4550 
//! Note that the SSPADD acts as a counter to determine the I2C frequency. A   
//! smaller value will increase the frequency.
//!
//! \param None
//!
//! \return None
//
//*****************************************************************************
void
Comms_init()
{
    TRISB = TRISB | 0x03;	//set TRISB<1:0> SDA and SCL
    TRISA = TRISA | 0x02;	//I2C ready line input

    PIR1bits.SSPIF = 0;		//clear I2C interrupt flag

    SSPADD = 0x1C;		//settings for I2C frequency - 416kHz
    SSPSTAT |= 0x80;		//slew rate control for high speed (400kHz)

    SSPCON1 = 0x28;		//enables I2C module on the PIC18F4550

    LATB = LATB | 0x04;		//IQS316 MCLR High
}

//*****************************************************************************
//
//! I2C Write
//!
//! A single byte is transmitted on the I2C bus. Note that the I2C communication 
//! channel must already be active before calling this,  
//! as no start bits are included in this function
//!
//! \param Byte to transmit to the master
//!
//! \return None
//
//*****************************************************************************
void
CommsIQS316_send(uint8_t send_data)
{
	SSPBUF = send_data;
	
	while (PIR1bits.SSPIF == 0)
	{}
	PIR1bits.SSPIF = 0;                 //clear flag

	while (SSPCON2bits.ACKSTAT == 1)    //verify IQS420 acknowledge
	{}
}

//*****************************************************************************
//
//! I2C Read + NACK
//!
//! A single byte is read from the I2C bus.  This is  the last byte in a read
//! sequence, because an NACK is sent to the master after reading, indicating 
//! there are no further bytes to follow.
//!
//! \param None
//!
//! \return Byte read from master
//
//*****************************************************************************
uint8_t
CommsIQS316_read_nack(void)
{
	unsigned char temp;
	
	SSPCON2bits.RCEN = 1;
	while (PIR1bits.SSPIF == 0)
	{}
	PIR1bits.SSPIF = 0;			//clear flag

	while (SSPSTATbits.BF == 0)
	{}
	temp = SSPBUF;

	SSPCON2bits.ACKDT = 1;
	SSPCON2bits.ACKEN = 1;

	while (PIR1bits.SSPIF == 0)
	{}
	PIR1bits.SSPIF = 0;			//clear flag
	
	while (SSPCON2bits.ACKEN == 1) {}

	return temp;
}

//*****************************************************************************
//
//! I2C Read + ACK
//!
//! A single byte is read from the I2C bus.  This is not the last byte in a read
//! sequence, because an ACK is sent to the master after reading, indicating 
//! another byte must follow.
//!
//! \param None
//!
//! \return Byte read from master
//
//*****************************************************************************
uint8_t
CommsIQS316_read_ack(void)
{
	unsigned char temp;
	
	SSPCON2bits.RCEN = 1;
	while (PIR1bits.SSPIF == 0)
	{}
	PIR1bits.SSPIF = 0;			//clear flag

	while (SSPSTATbits.BF == 0)
	{}
	temp = SSPBUF;

	SSPCON2bits.ACKDT = 0;
	SSPCON2bits.ACKEN = 1;

	while (PIR1bits.SSPIF == 0)
	{}
	PIR1bits.SSPIF = 0;			//clear flag
	
	while (SSPCON2bits.ACKEN == 1) {}

	return temp;
}

//*****************************************************************************
//
//! I2C Start
//!
//! A start is generated on the I2C bus. Since a START is the first i2c 
//! transaction at the beginning of a comms window, the RDY line is checked.
//! Comms is only begun once RDY is active (high)
//!
//! \param None
//!
//! \return None
//
//*****************************************************************************
void
CommsIQS316_start(void)
{
	while (RDY == 0)		//wait for ready
	{}

	SSPCON2bits.SEN = 1;		//start condition

	while (PIR1bits.SSPIF == 0)	//wait for start condition to be generated
	{}
	PIR1bits.SSPIF = 0;		//clear flag

	while (SSPCON2bits.SEN == 1)
	{}
}

//*****************************************************************************
//
//! I2C STOP
//!
//! A standard I2C STOP condition is generated on the i2c bus.
//!
//! \param None
//!
//! \return None
//
//*****************************************************************************
void
CommsIQS316_stop(void)
{
	SSPCON2bits.PEN = 1;		//stop condition

	while (PIR1bits.SSPIF == 0)	//wait for stop condition to be generated
	{}
	PIR1bits.SSPIF = 0;		//clear flag

	while (SSPCON2bits.PEN == 1)
	{}
}

// =============================================================================
// =========== PURELY FOR DEBUGGING - not part of library ======================

void
DelayMS(uint8_t ui8Milliseconds)
{
    uint16_t ui16i, ui16Count;
    uint16_t ui16LoopLength;

    ui16LoopLength = 500*(uint16_t)ui8Milliseconds;

    for(ui16i = 0 ; ui16i < ui16LoopLength ; ui16i++)
    {
        ui16Count++ ;
    }
}

void
DebugLED(uint8_t ui8LEDState, uint8_t ui8LEDNumber)
{
   if(ui8LEDState == CLEAR_LED)
   {
      // LED is inactive on an I/O HIGH state (sink)
      //
      LATD |= ui8LEDNumber;
   }
   else
   {
       // LED is active on an I/O LOW state (sink)
      LATD &= (uint8_t)~ui8LEDNumber;
   }
}

void
DebugIO(uint8_t ui8IOState, uint8_t ui8IONumber)
{
   if(ui8IOState == SET_IO)
   {
      LATC |= ui8IONumber;
   }
   else
   {
      LATC &= (uint8_t)~ui8IONumber;
   }
}

void
PulseIO(uint8_t ui8IONumber, uint8_t ui8NoOfPulses)
{
    uint8_t ui8i;

    for(ui8i = 0 ; ui8i < ui8NoOfPulses ; ui8i++)
    {
        LATD |= ui8IONumber;
        LATD &= (uint8_t)~ui8IONumber;
    }
}
